// OggMuxDlg.cpp : Implementation File

/*****************************************************************************
 *
 *  OggMux
 *  - Front-end for Tobias Waldvogel's OGG directshow filters -
 *
 *  This program implements a GUI for those filters to multiplex different
 *  stream types (audio, video, subtitles, chapterinformation,...) into
 *  the OGG container format.
 *
 *  This program is free software ; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation ; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY ; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program ; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 ****************************************************************************/

/*****************************************************************************
 *  History:
 *
 *	v 0.9.4.1: Fixed support for movieinfo in automisation mode
 *  v 0.9.4: Added copyright headers to the sources (Koepi)
 *
 *  All code is (c)2002 by Dirk Knop, no further code has made it into
 *  OggMux yet.
 *
 ****************************************************************************/

#include "stdafx.h"
#include "OggMux.h"
#include "MovieInfo.h"
#include "OggMuxDlg.h"
#include <iostream.h>
#include <fstream.h>
// new includes necessary for building filtergraphs
#include <streams.h>
#include <initguid.h>
#include "OggDS.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
/////////////////////////////////////////////////////////////////////////////
// enable to register the filtergraph globally
/////////////////////////////////////////////////////////////////////////////
//#define REGISTER_FILTERGRAPH

// Global variables
CString		TargetFile;
int			UsedPins;
CStringList	SndTrk, SndTrkLng, SndTrkDelay, SndTrkQual,
			Subt, SubtLng, SubtDelay, ChapterName, ChapterTime;

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg-Dialogfield for "Info"

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialogfield data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CStatic	m_static_url;
	//}}AFX_DATA

	// class wizard generated overloads of virtual functions
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV-support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual void OnOK();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COggMuxDlg Dialogfield

COggMuxDlg::COggMuxDlg(CWnd* pParent /*=NULL*/)
	: CDialog(COggMuxDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(COggMuxDlg)
	m_soundtrack1 = _T("");
	m_targetfile = _T("");
	m_movie = _T("");
	m_language1 = _T("(not specified)");
	m_language2 = _T("(not specified)");
	m_subtitles1 = _T("");
	m_quality = 1;
	m_soundtrack_delay = 0;
	m_subtitles_delay = 0;
	m_hint1 = _T("After choosing a soundtrack or subtitle, select the language and set the encoding parameters according to your desired values.");
	m_hint2 = _T("Don't forget to use the 'Add'-button afterwards! For adding an ac3-wav set quality to 0! Use BeSweet to create an ac3-wav.");
	m_static_position = _T("");
	m_static_position2 = _T("");
	m_chaptername = _T("");
	m_chaptertime = _T("");
	m_splitsize = 0;
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void COggMuxDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COggMuxDlg)
	DDX_Control(pDX, IDC_B_RESET, m_resetbutton);
	DDX_Control(pDX, IDC_B_CLEAR_CHAPTERLIST, m_clear_chapterlist);
	DDX_Control(pDX, IDC_B_CHAPTER_MODIFY, m_chapter_modify);
	DDX_Control(pDX, IDC_B_CHAPTER_DELETE, m_chapter_delete);
	DDX_Control(pDX, IDC_LIST_CHAPTERS, m_chapterlist);
	DDX_Control(pDX, IDC_B_SUBTITLE_MODIFY, m_subtitles_modify);
	DDX_Control(pDX, IDC_B_SUBTITLE_DELETE, m_subtitles_delete);
	DDX_Control(pDX, IDC_B_SOUNDTRACK_MODIFY, m_soundtrack_modify);
	DDX_Control(pDX, IDC_B_SOUNDTRACK_DELETE, m_soundtrack_delete);
	DDX_Control(pDX, IDC_PROGRESSBAR, m_progress);
	DDX_Control(pDX, IDC_E_QUALITY, m_edit_quality);
	DDX_Control(pDX, IDC_STATIC_QUALITY, m_static_quality);
	DDX_Control(pDX, IDC_B_MUXFILE, m_muxbutton);
	DDX_Control(pDX, IDC_LIST_SUBTITLES, m_subtitleslist);
	DDX_Control(pDX, IDC_LIST_SOUNDTRACKS, m_soundtracklist);
	DDX_Text(pDX, IDC_E_SOUNDTRACK1, m_soundtrack1);
	DDX_Text(pDX, IDC_E_TARGETFILE, m_targetfile);
	DDX_Text(pDX, IDC_E_MOVIE, m_movie);
	DDX_CBString(pDX, IDC_COMBO_LANGUAGE1, m_language1);
	DDX_CBString(pDX, IDC_COMBO_LANGUAGE2, m_language2);
	DDX_Text(pDX, IDC_E_SUBTITLES1, m_subtitles1);
	DDX_Text(pDX, IDC_E_QUALITY, m_quality);
	DDV_MinMaxInt(pDX, m_quality, 0, 1000);
	DDX_Text(pDX, IDC_E_SOUNDTRACKDELAY, m_soundtrack_delay);
	DDV_MinMaxInt(pDX, m_soundtrack_delay, -10000, 10000);
	DDX_Text(pDX, IDC_E_SUBTITLESDELAY, m_subtitles_delay);
	DDV_MinMaxInt(pDX, m_subtitles_delay, -10000, 10000);
	DDX_Text(pDX, IDC_STATIC_HINT1, m_hint1);
	DDX_Text(pDX, IDC_STATIC_HINT2, m_hint2);
	DDX_Text(pDX, IDC_STATIC_POSITION, m_static_position);
	DDX_Text(pDX, IDC_STATIC_POSITION2, m_static_position2);
	DDX_Text(pDX, IDC_E_CHAPTERNAME, m_chaptername);
	DDX_Text(pDX, IDC_E_CHAPTERTIME, m_chaptertime);
	DDX_CBIndex(pDX, IDC_COMBO_SPLIT, m_splitsize);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(COggMuxDlg, CDialog)
	//{{AFX_MSG_MAP(COggMuxDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_B_MUXFILE, OnBMuxfile)
	ON_BN_CLICKED(IDC_B_OPENMOVIE, OnBOpenmovie)
	ON_BN_CLICKED(IDC_B_OPENSOUNDTRACK1, OnBOpensoundtrack1)
	ON_BN_CLICKED(IDC_B_SAVETARGETFILE, OnBSavetargetfile)
	ON_BN_CLICKED(IDC_B_OPENSUBTITLES1, OnBOpensubtitles1)
	ON_CBN_SELCHANGE(IDC_COMBO_LANGUAGE1, OnSelchangeComboLanguage1)
	ON_CBN_SELCHANGE(IDC_COMBO_LANGUAGE2, OnSelchangeComboLanguage2)
	ON_BN_CLICKED(IDC_B_ADDSOUNDTRACK, OnBAddsoundtrack)
	ON_BN_CLICKED(IDC_B_ADDSUBTITLES, OnBAddsubtitles)
	ON_LBN_SELCHANGE(IDC_LIST_SOUNDTRACKS, OnSelchangeListSoundtracks)
	ON_LBN_SELCHANGE(IDC_LIST_SUBTITLES, OnSelchangeListSubtitles)
	ON_EN_UPDATE(IDC_E_SOUNDTRACK1, OnUpdateESoundtrack1)
	ON_BN_CLICKED(IDC_B_RESET, OnBReset)
	ON_BN_CLICKED(IDC_B_ABOUT, OnBAbout)
	ON_BN_CLICKED(IDC_B_SOUNDTRACK_MODIFY, OnBSoundtrackModify)
	ON_BN_CLICKED(IDC_B_SOUNDTRACK_DELETE, OnBSoundtrackDelete)
	ON_BN_CLICKED(IDC_B_SUBTITLE_MODIFY, OnBSubtitleModify)
	ON_BN_CLICKED(IDC_B_SUBTITLE_DELETE, OnBSubtitleDelete)
	ON_BN_CLICKED(IDC_B_ADDCHAPTER, OnBAddchapter)
	ON_BN_CLICKED(IDC_B_CHAPTER_DELETE, OnBChapterDelete)
	ON_BN_CLICKED(IDC_B_CHAPTER_MODIFY, OnBChapterModify)
	ON_LBN_SELCHANGE(IDC_LIST_CHAPTERS, OnSelchangeListChapters)
	ON_BN_CLICKED(IDC_B_IMPORT_CHAPTERS, OnBImportChapters)
	ON_BN_CLICKED(IDC_B_CLEAR_CHAPTERLIST, OnBClearChapterlist)
	ON_BN_CLICKED(IDC_MOVIEINFO, OnMovieinfo)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COggMuxDlg Message-Handler

BOOL COggMuxDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{	
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	SetIcon(m_hIcon, TRUE);

// Adding ToolTips
	m_ToolTip.Create (this);
	m_ToolTip.Activate (TRUE);

	CWnd*    pWnd = GetWindow (GW_CHILD);
	while (pWnd)
	{
		int nID = pWnd->GetDlgCtrlID ();
		if (nID != -1)
	    {
		    m_ToolTip.AddTool (pWnd, pWnd->GetDlgCtrlID ());
		}
		pWnd = pWnd->GetWindow (GW_HWNDNEXT);
		running = false;
	}
// End of adding ToolTips

	CString m_commandline;
	automate = false;
// automisation via -f command line switch
	m_commandline=AfxGetApp()->m_lpCmdLine;
	if (m_commandline.Left(2)=="-f" && m_commandline.Right(3)=="omx") 
	{
		automate = true;
		automisation(m_commandline.Right(m_commandline.GetLength()-3));
	}
// automisation via calling app with a filename
	if (m_commandline != "" && m_commandline.Right(3)=="omx") 
	{
		automate = true;
		automisation(m_commandline.Right(m_commandline.GetLength()));
	}

	return TRUE;
}

void COggMuxDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

void COggMuxDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this);

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

HCURSOR COggMuxDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

////////////////////////////////////////////////////////////
// Functions necessary for accessing the graph via graphedit
////////////////////////////////////////////////////////////
HRESULT COggMuxDlg::AddToRot(IUnknown *pUnkGraph, DWORD *pdwRegister) 
{
    IMoniker			*pMoniker;
    IRunningObjectTable	*pROT;
    if (FAILED(GetRunningObjectTable(0, &pROT))) {
        return E_FAIL;
    }
    WCHAR				wsz[256];
	char				temp[256];
    wsprintf(temp, "FilterGraph %08p pid %08x", (DWORD_PTR)pUnkGraph, GetCurrentProcessId());
	mbstowcs(wsz, temp, 257);
    HRESULT hr = CreateItemMoniker(L"!", wsz, &pMoniker);
    if (SUCCEEDED(hr)) {
        hr = pROT->Register(0, pUnkGraph, pMoniker, pdwRegister);
        pMoniker->Release();
    }
    pROT->Release();
    return hr;
}

void COggMuxDlg::RemoveFromRot(DWORD pdwRegister)
{
	IRunningObjectTable		*pROT;
    if (SUCCEEDED(GetRunningObjectTable(0, &pROT))) {
        pROT->Revoke(pdwRegister);
        pROT->Release();
    }
}

////////////////////////////////////////////////////////////
// Create a graph builder to mux the ogg media file
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBMuxfile() 
{
	HRESULT					hr;
	IGraphBuilder			*pGraph = NULL;
	IBaseFilter				*pOggMux = NULL;
	IMediaEvent				*pMediaEvent = NULL;
	IMediaControl			*pMediaControl = NULL;
	IMediaSeeking			*pMediaSeeking = NULL;
	IBaseFilter				*pTargetFile = NULL;
	IFileSinkFilter2		*FileDest=NULL;
	WCHAR					transformed[MAX_PATH];

	IPin					*inPin = NULL;
	IPin					*outPin = NULL;
	IPropertyBag			*pPropBag = NULL;

#ifdef REGISTER_FILTERGRAPH
	DWORD					RegRot;
#endif
	CString					FindPin, my_language, my_file;

	UpdateData(true);

	if (!running) {

	CoInitialize(NULL);

	UsedPins = 0;	// Used Input Pins on the OggMuxer

	if (m_targetfile=="")
		MessageBox("No target file selected, aborting.","Error!",MB_OK|MB_ICONEXCLAMATION);
	else
	{
////////////////////////////////////////////////////////////
// Create a filter graph
////////////////////////////////////////////////////////////

// Create instance of the FilterGraphBuilder
		hr = CoCreateInstance(CLSID_FilterGraph,	// CLSID of object
		    NULL,									// outer unknown
			CLSCTX_INPROC_SERVER,					// type of server
			IID_IGraphBuilder,						// interface wanted
			(void **) &pGraph);						// pointer to IGraphBuilder
		if (FAILED(hr))
			MessageBox("Can't create a filtergraph!", "Error!", MB_OK);

// Register FilterGraph globally (for connection via GraphEdit)
#ifdef REGISTER_FILTERGRAPH
	AddToRot(pGraph, &RegRot);
#endif

////////////////////////////////////////////////////////////
// Create Output file filter
////////////////////////////////////////////////////////////

// convert Filename to WCHAR
		mbstowcs(transformed, m_targetfile, m_targetfile.GetLength()+1);
// Create instance of the FileWriter Filter
		hr = CoCreateInstance(CLSID_FileWriter,		// CLSID of object
		    NULL,									// outer unknown
			CLSCTX_INPROC_SERVER,					// type of server
			IID_IBaseFilter,						// interface wanted
			(void **) &pTargetFile);				// pointer to FileWriter
		if (FAILED(hr))
			MessageBox("Can't create FileWriter!", "Error!", MB_OK);
// Get the FileSink interface for setting the filename
		hr = pTargetFile->QueryInterface(IID_IFileSinkFilter2, (void**)&FileDest);
		if (FAILED(hr))
			MessageBox("Can't queryinterface FileDest on TargetFile!", "Error!", MB_OK);
// Set the Filename
		hr = FileDest->SetFileName(transformed,NULL);
		if (FAILED(hr))
			MessageBox("Can't SetFileName at FileDest!", "Error!", MB_OK);
// Set File to overwrite existing by deleting them
		hr = FileDest->SetMode(AM_FILE_OVERWRITE);
// Add Filter to the Filtergraph
		hr = pGraph->AddFilter(pTargetFile, L"File Writer");
		if (FAILED(hr))
			MessageBox("Can't add FileWriter to Graph!", "Error!", MB_OK);
		FileDest->Release();

////////////////////////////////////////////////////////////
// Enumerate pins to connect the FileDest
////////////////////////////////////////////////////////////
		hr = pTargetFile->FindPin(L"in", &inPin);
		if (FAILED(hr))
			MessageBox("Can't find FileWriter input pin!", "Error!", MB_OK);

////////////////////////////////////////////////////////////
// Create Muxer file filter
////////////////////////////////////////////////////////////
// Create an instance of the OggMuxer Filter
		hr = CoCreateInstance(CLSID_OggMux,		// CLSID of object
			NULL,								// outer unknown
			CLSCTX_INPROC_SERVER,				// type of server
			IID_IBaseFilter,					// interface wanted
			(void **) &pOggMux);				// pointer to Ogg Muxer
		if (FAILED(hr))
			MessageBox("Can't create OggMuxer!", "Error!", MB_OK);
// Add OggMuxer Filter to the Filtergraph
		hr = pGraph->AddFilter(pOggMux, L"Ogg Multiplexer");
		if (FAILED(hr))
			MessageBox("Can't add OggMuxer to Graph!", "Error!", MB_OK);

////////////////////////////////////////////////////////////
// Enumerate pins to connect the OggMuxer
////////////////////////////////////////////////////////////
		hr = pOggMux->FindPin(L"Ogg Stream", &outPin);
		if (FAILED(hr))
			MessageBox("Can't find oggmux output pin!", "Error!", MB_OK);

////////////////////////////////////////////////////////////
// Connect the OggMuxer with the FileWriter
////////////////////////////////////////////////////////////
		hr = pGraph->Connect(outPin, inPin);
		if (FAILED(hr))
			MessageBox("Can't connect OggMuxer output with FileWriter input!", "Error!", MB_OK);

		inPin->Release();
		outPin->Release();

////////////////////////////////////////////////////////////
// Add input sources
////////////////////////////////////////////////////////////
// Add source for Movie
		if (m_movie != "") {
			RenderFile(m_movie, "Moviesource", "Title", m_movieinfo.m_title, pGraph, pOggMux);
			InsertChapters(pOggMux);
		}

// Add sources for Soundtracks
		if (m_soundtracklist.GetCount() > 0)
		{
			for (int c=0; c<m_soundtracklist.GetCount(); c++) {
				set_soundtrack(c);
				if ((m_soundtrack1.Right(4)==".wav") && (m_quality > 0))
					EncodeFile(m_soundtrack1, m_quality/100, "Soundtracksource", "Language", m_language1, pGraph, pOggMux);
				else RenderFile(m_soundtrack1, "Soundtracksource", "Language", m_language1, pGraph, pOggMux);
			}
			m_soundtrack1 = "";
			m_language1 = "(not specified)";
			m_soundtrack_delay = 0;
			m_quality = 1;
			UpdateData(false);
		} else
			MessageBox("No soundtrack in list! Use the Add-button to add a soundtrack.", "No soundtrack error...", MB_OK|MB_ICONEXCLAMATION);

// Add sources for Subtitles
		if (m_subtitleslist.GetCount() > 0)
		{
			for (int c=0; c<m_subtitleslist.GetCount(); c++) {
				set_subtitles(c);
				RenderFile(m_subtitles1, "Subtitlesource", "Language", m_language2, pGraph, pOggMux);
			}
			m_subtitles1 = "";
			m_language2 = "(not specified)";
			m_subtitles_delay = 0;
			UpdateData(false);
		}

////////////////////////////////////////////////////////////
// run the graph
////////////////////////////////////////////////////////////
// Get the interface of the MediaControl
		hr = pGraph->QueryInterface(IID_IMediaControl, (void **) &pMediaControl);
		if (FAILED(hr))
			MessageBox("Can't query interface the media control!", "Error!", MB_OK);

// Query MediaEvent Interface
		hr = pGraph->QueryInterface(IID_IMediaEvent, (void **) &pMediaEvent);
		if (FAILED(hr))
			MessageBox("Can't query MediaEvent interface!", "Error!", MB_OK);

// ..and run, run, run the FilterGraph :)
		hr = pMediaControl->Run();
		if (FAILED(hr))
			MessageBox("Can't run the filtergraph!", "Error!", MB_OK);
		running = true;
		m_muxbutton.SetWindowText("Stop");

////////////////////////////////////////////////////////////
// Progress bar
////////////////////////////////////////////////////////////
		hr = pOggMux->FindPin(L"Ogg Stream", &outPin);
		if (FAILED(hr))
			MessageBox("Can't find OggMuxer outpin for progress bar!", "Error!", MB_OK);
		hr = outPin->QueryInterface(IID_IMediaSeeking, (void**)&pMediaSeeking);
		if (FAILED(hr))
			MessageBox("Can't use OggMuxer outpin media seeking for progress bar!", "Error!", MB_OK);
		else {
			pMediaSeeking->SetTimeFormat(&TIME_FORMAT_MEDIA_TIME);
			REFERENCE_TIME rtTotal, rtNow, m_Pos, rtKeyframe, rtTemp, bytes = 0;
			int period = 100;
			long evCode;
			char temp[100];
			int parts = 1;
			int part_bytes = 0;

			switch (m_splitsize) {
			case 1:
				part_bytes=650*1024*1024;
				break;
			case 2:
				part_bytes=700*1024*1024;
				break;
			case 3:
				part_bytes=800*1024*1024;
				break;
			case 4:
				part_bytes=900*1024*1024;
				break;
			case 5:
				part_bytes=998*1024*1024;
				break;
			default:
				part_bytes=0;
			}
			
			hr = pMediaSeeking->GetDuration(&rtTotal);
			wsprintf(temp, "Length: %2.2i:%2.2i:%2.2i.%3.3i",
		        (int)(rtTotal / 36000000000),        // hours
				(int)(rtTotal /   600000000 % 60),   // minutes
				(int)(rtTotal /    10000000 % 60),   // seconds
				(int)(rtTotal /       10000 % 1000)  // msecs
				);
			rtTotal++;
			m_static_position2=temp;
			if (FAILED(hr))
				MessageBox("Can't get OggMuxer media seeking duration!", "Error!", MB_OK);
			hr = pMediaSeeking->GetCurrentPosition(&rtNow);
			m_progress.SetRange32((REFERENCE_TIME)(rtNow / period),(REFERENCE_TIME)(rtTotal/period)+1);
			rtTotal-=10000000;
			while (running && (pMediaEvent->WaitForCompletion(period, &evCode)) == E_ABORT)	{
				pMediaSeeking->SetTimeFormat(&TIME_FORMAT_MEDIA_TIME);
		        pMediaSeeking->GetCurrentPosition(&rtNow);
				m_Pos = (REFERENCE_TIME)(rtNow/period);
				m_progress.SetPos(m_Pos);
				wsprintf(temp, "%2.2i%%", (int)(rtNow*100/rtTotal));
				m_muxbutton.SetWindowText("Stop");
				m_resetbutton.SetWindowText(temp);
				m_progress.UpdateData(false);
				wsprintf(temp, "Position: %2.2i:%2.2i:%2.2i.%3.3i",
			        (int)(rtNow / 36000000000),        // hours
					(int)(rtNow /   600000000 % 60),   // minutes
					(int)(rtNow /    10000000 % 60),   // seconds
					(int)(rtNow /       10000 % 1000)  // msecs
					);
				m_static_position=temp;

////////////////////////////////////////////////////////////
// Process Windows messages
////////////////////////////////////////////////////////////
	          MSG msg;
				if(PeekMessage(&msg, AfxGetMainWnd()->m_hWnd,0, 0, PM_REMOVE))
				{ /* handle it*/
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}/* handle it */

////////////////////////////////////////////////////////////
// Split output files
////////////////////////////////////////////////////////////
				if (part_bytes > 0)
				{
			        pMediaSeeking->SetTimeFormat(&TIME_FORMAT_BYTE);
					pMediaSeeking->GetCurrentPosition(&bytes);
					wsprintf(temp, "%3.3i MB - Part %2.2i", (int)(bytes/(1024*1024)), parts);
					m_static_position2=temp;
					if (bytes >= part_bytes)
					{
// Stop the FilterGraph
						pMediaControl->Stop();
// Find a keyframe
						pMediaSeeking->SetTimeFormat(&TIME_FORMAT_MEDIA_TIME);
						pMediaSeeking->GetPositions(&rtNow,&rtTotal);
						rtKeyframe=rtNow-1200000000; // 2 minutes back...
						while (rtKeyframe < rtNow) {
							rtTemp=rtKeyframe;
							pMediaSeeking->SetPositions(&rtKeyframe,AM_SEEKING_SeekToKeyFrame, &rtTotal, AM_SEEKING_NoPositioning);
							pMediaSeeking->GetPositions(&rtKeyframe,&rtTotal);
							if (rtTemp==rtKeyframe)  // prevent deadlock
								rtKeyframe+=4000000; // ~1 frame forward
							pMediaSeeking->SetPositions(&rtKeyframe, AM_SEEKING_AbsolutePositioning, &rtTotal, AM_SEEKING_NoPositioning);
							m_static_position2="Seeking keyframe...";
							UpdateData(false);
						}
						pMediaSeeking->GetPositions(&rtKeyframe,&rtTotal);
						pMediaSeeking->SetPositions(&rtKeyframe, AM_SEEKING_AbsolutePositioning, &rtTotal, AM_SEEKING_NoPositioning);

// Set new output filename
						parts++;
						CString tempname;
						char temp2[MAX_PATH];
						wsprintf(temp2, "%s-%i%s", m_targetfile.Left(m_targetfile.GetLength()-4), parts,m_targetfile.Right(4));
						tempname=temp2;
						mbstowcs(transformed, tempname, tempname.GetLength()+1);
// Get the FileSink interface for setting the filename
						hr = pTargetFile->QueryInterface(IID_IFileSinkFilter2, (void**)&FileDest);
						if (FAILED(hr))
							MessageBox("Can't queryinterface FileDest on TargetFile!", "Error!", MB_OK);
// Set the Filename
						hr = FileDest->SetFileName(transformed,NULL);
						if (FAILED(hr))
							MessageBox("Can't SetFileName at new FileDest!", "Error!", MB_OK);
						pTargetFile->Release();
						FileDest->Release();
				
// Restart the FilterGraph
						hr = pMediaControl->Run();
						if (FAILED(hr))
							MessageBox("Can't continue the filtergraph!", "Error!", MB_OK);
					}  // rtNow>partbytes
				}  // splitting active
				UpdateData(false);
				UpdateWindow();
			}
		}
		outPin->Release();

////////////////////////////////////////////////////////////
// ...and now, clean up all the mess:
////////////////////////////////////////////////////////////
// Remove FilterGraph from global registration
#ifdef REGISTER_FILTERGRAPH
	RemoveFromRot(RegRot);
#endif

		pMediaControl->Release();
		pMediaEvent->Release();
		pMediaSeeking->Release();
		pOggMux->Release();
		pTargetFile->Release();
		pGraph->Release();
		running = false;
		m_resetbutton.SetWindowText("Reset");
		m_muxbutton.SetWindowText("Mux File!");
	}  // if m_tagretfile...
	CoUninitialize();
	}  // if !running
	else {
		running = false;
	} 
}

////////////////////////////////////////////////////////////
// Add Vorbis Encoder Filter to the FilterGraph
////////////////////////////////////////////////////////////
void COggMuxDlg::EncodeFile(CString Filename, int Quality, CString Filtername, CString Property, CString Value, IGraphBuilder *Graph, IBaseFilter *OggMuxer)
{
	IBaseFilter				*VorbisEncoder = NULL;
	IPin					*inPin = NULL;
	IPin					*outPin = NULL;
	IPropertyBag			*pPropBag = NULL;
	IAMStreamConfig			*pStreamConfig = NULL;
	AM_MEDIA_TYPE			*pmt = NULL;
	WCHAR					transformed[MAX_PATH];
	WCHAR					transfilter[MAX_PATH];
	char					tempstring[400];
	CString					FindPin="";
	HRESULT					hr;
	CString					ErrMsg="";

////////////////////////////////////////////////////////////
// Create Vorbis encoder filter
////////////////////////////////////////////////////////////

// Create an instance of the Vorbis encoder Filter
	hr = CoCreateInstance(CLSID_VorbisEnc,	// CLSID of object
		NULL,								// outer unknown
		CLSCTX_INPROC_SERVER,				// type of server
		IID_IBaseFilter,					// interface wanted
		(void **) &VorbisEncoder);			// pointer to Vorbis encoder
	if (FAILED(hr))
		MessageBox("Can't create Vorbis Encoder!", "Error!", MB_OK);
// Add Vorbis encoder Filter to the Filtergraph
	hr = Graph->AddFilter(VorbisEncoder, L"Vorbis Compressor");
	if (FAILED(hr))
		MessageBox("Can't add Vorbis encoder to Graph!", "Error!", MB_OK);

////////////////////////////////////////////////////////////
// Render the wav
////////////////////////////////////////////////////////////
// Convert File-&Filtername to WCHAR
	mbstowcs(transformed, Filename, Filename.GetLength()+1); 
	mbstowcs(transfilter, Filtername, Filtername.GetLength()+1);
// Let the FilterGraphBuilder render the file
	hr = Graph->RenderFile(transformed, transfilter);
// Create Error Message
	wsprintf(tempstring, "Can't render input file %s!", Filename);
	ErrMsg=tempstring;
	if (FAILED(hr))
		MessageBox(ErrMsg, "Render File Error!", MB_OK);

////////////////////////////////////////////////////////////
// Enumerate pins and setup the Vorbis Encoder
////////////////////////////////////////////////////////////
	hr = VorbisEncoder->FindPin(L"Out", &outPin);
	if (FAILED(hr))
		MessageBox("Can't find Vorbis encoder output pin!", "Error!", MB_OK);
	else {
		hr = outPin->QueryInterface(IID_IAMStreamConfig, (void**) &pStreamConfig);
		pStreamConfig->GetFormat(&pmt);
		((VORBISFORMAT*)(pmt->pbFormat))->nMinBitsPerSec = -1;
		((VORBISFORMAT*)(pmt->pbFormat))->nAvgBitsPerSec = -1;
		((VORBISFORMAT*)(pmt->pbFormat))->nMaxBitsPerSec = -1;
		((VORBISFORMAT*)(pmt->pbFormat))->fQuality = (float)((float)(Quality) / (float)100);
		pStreamConfig->SetFormat(pmt);
		pStreamConfig->Release();
		CoTaskMemFree(pmt->pbFormat);
		CoTaskMemFree(pmt);
	}

// Find the input pin where added source to OggMuxer
	wsprintf(tempstring, "Stream %i", UsedPins);
	FindPin = tempstring;
	mbstowcs(transformed, FindPin, FindPin.GetLength()+1); 
	hr = OggMuxer->FindPin(transformed, &inPin);
	if (FAILED(hr))
		MessageBox("Can't find OggMuxer input pin!", "Render File Error!", MB_OK);
// Convert language string
	mbstowcs(transfilter, Property, Property.GetLength()+1);
	mbstowcs(transformed, Value, Value.GetLength()+1); 
	VARIANT	V;
	V.vt = VT_BSTR;
	V.bstrVal = SysAllocString(transformed);
// Get the OggMuxer property interface of the pin
	inPin->QueryInterface(IID_IPropertyBag, (void**)&pPropBag);
// Finally write the language of the pin
	pPropBag->Write(transfilter, &V);
// Clean up the stuff
	VariantClear(&V);
	pPropBag->Release();
	UsedPins++;
// Clean up
	inPin->Release();
	outPin->Release();
	VorbisEncoder->Release();
}

////////////////////////////////////////////////////////////
// Add File Render Filter to the FilterGraph
////////////////////////////////////////////////////////////
void COggMuxDlg::RenderFile(CString Filename, CString Filtername, CString Property, CString Value, IGraphBuilder *Graph, IBaseFilter *OggMuxer)
{
	IPin					*inPin = NULL;
	IPropertyBag			*pPropBag = NULL;
	WCHAR					transformed[MAX_PATH];
	WCHAR					transfilter[MAX_PATH];
	char					tempstring[400];
	CString					FindPin="";
	HRESULT					hr;
	CString					ErrMsg="";

// Convert File-&Filtername to WCHAR
		mbstowcs(transformed, Filename, Filename.GetLength()+1); 
		mbstowcs(transfilter, Filtername, Filtername.GetLength()+1);
// Let the FilterGraphBuilder render the file
		hr = Graph->RenderFile(transformed, transfilter);
// Create Error Message
		wsprintf(tempstring, "Can't render input file %s!", Filename);
		ErrMsg=tempstring;
		if (FAILED(hr))
			MessageBox(ErrMsg, "Render File Error!", MB_OK);
// Find the input pin where added source to OggMuxer
		wsprintf(tempstring, "Stream %i", UsedPins);
		FindPin = tempstring;
		mbstowcs(transformed, FindPin, FindPin.GetLength()+1); 
		hr = OggMuxer->FindPin(transformed, &inPin);
		if (FAILED(hr))
			MessageBox("Can't find OggMuxer input pin!", "Render File Error!", MB_OK);
// Convert language string
		mbstowcs(transfilter, Property, Property.GetLength()+1);
		mbstowcs(transformed, Value, Value.GetLength()+1); 
		VARIANT	V;
		V.vt = VT_BSTR;
		V.bstrVal = SysAllocString(transformed);
// Get the OggMuxer property interface of the pin
		inPin->QueryInterface(IID_IPropertyBag, (void**)&pPropBag);
// Finally write the language of the pin
		pPropBag->Write(transfilter, &V);
// Clean up the stuff
		VariantClear(&V);
		if (Property=="Title")  // movie source, add infos
		{
			mbstowcs(transfilter, "Author", 11);
			mbstowcs(transformed, m_movieinfo.m_author, m_movieinfo.m_author.GetLength()+1); 
			V.vt = VT_BSTR;
			V.bstrVal = SysAllocString(transformed);
			pPropBag->Write(transfilter, &V);
			VariantClear(&V);

			mbstowcs(transfilter, "Copyright", 10);
			mbstowcs(transformed, m_movieinfo.m_copyright, m_movieinfo.m_copyright.GetLength()+1); 
			V.vt = VT_BSTR;
			V.bstrVal = SysAllocString(transformed);
			pPropBag->Write(transfilter, &V);
			VariantClear(&V);

			mbstowcs(transfilter, "Description", 12);
			mbstowcs(transformed, m_movieinfo.m_description, m_movieinfo.m_description.GetLength()+1); 
			V.vt = VT_BSTR;
			V.bstrVal = SysAllocString(transformed);
			pPropBag->Write(transfilter, &V);
			VariantClear(&V);
		}
		inPin->Release();
		pPropBag->Release();
		UsedPins++;
}

////////////////////////////////////////////////////////////
// Insert Chapters to current Muxer input pin
////////////////////////////////////////////////////////////
void COggMuxDlg::InsertChapters(IBaseFilter *OggMuxer)
{
	IPin					*inPin = NULL;
	IPropertyBag			*pPropBag = NULL;
	WCHAR					transformed[400];
	WCHAR					transformed2[400];
	char					tempstring[400];
	CString					FindPin,temp="";
	HRESULT					hr;
	CString					ErrMsg="";

// Find the input pin where added source to OggMuxer
	FindPin = "Stream 0";
	mbstowcs(transformed, FindPin, FindPin.GetLength()+1); 
	hr = OggMuxer->FindPin(transformed, &inPin);
	if (FAILED(hr))
		MessageBox("Can't find OggMuxer input pin!", "Render File Error!", MB_OK);
// Get the OggMuxer property interface of the pin
	inPin->QueryInterface(IID_IPropertyBag, (void**)&pPropBag);
// insert the list
	if (ChapterName.GetCount()>0) {
		for (int c=0; c<ChapterName.GetCount();c++) {
			set_chapter(c);
// Convert chapter time and write it to the inPin
			c<9?wsprintf(tempstring, "CHAPTER0%i", (c+1)):wsprintf(tempstring, "CHAPTER%i", (c+1));
			temp=tempstring;
			mbstowcs(transformed, temp, temp.GetLength()+1);
			temp=ChapterTime.GetAt(get_chaptertime_pos(c));
			mbstowcs(transformed2, temp, temp.GetLength()+1);
			VARIANT	V;
			V.vt = VT_BSTR;
			V.bstrVal = SysAllocString(transformed2);
			hr = pPropBag->Write(transformed, &V);
			if (FAILED(hr))
				MessageBox("Can't write chaptertime to OggMuxer pin!", "Chapter Write Error!", MB_OK);
			VariantClear(&V);
// Convert chapter name and write it to the inPin
			c<9?wsprintf(tempstring, "CHAPTER0%iNAME", (c+1)):wsprintf(tempstring, "CHAPTER%iNAME", (c+1));
			temp=tempstring;
			mbstowcs(transformed, temp, temp.GetLength()+1);
			temp=ChapterName.GetAt(get_chaptername_pos(c));
			mbstowcs(transformed2, temp, temp.GetLength()+1);
			V.vt = VT_BSTR;
			V.bstrVal = SysAllocString(transformed2);
			hr = pPropBag->Write(transformed, &V);
			if (FAILED(hr))
				MessageBox("Can't write chaptername to OggMuxer pin!", "Chapter Write Error!", MB_OK);
			VariantClear(&V);
		}  // of for...
	}  // of if...
// Clean up the stuff
	inPin->Release();
	pPropBag->Release();
	m_chaptername="";
	m_chaptertime="";
	UpdateData(false);
}

////////////////////////////////////////////////////////////
// Open / Save File Dialogs
////////////////////////////////////////////////////////////
void COggMuxDlg::FetchFile(CString* Target, CString Filetype)
{
	char temp[MAX_PATH];
	UpdateData(TRUE);	
 	CFileDialog OpenFile(true, NULL, NULL, NULL, Filetype, NULL);
	if (OpenFile.DoModal()==IDOK) {
		strcpy(temp, OpenFile.GetPathName());
		Target->Insert(0, temp);
		UpdateData(FALSE);
	}
}

void COggMuxDlg::OnBOpenmovie() 
{
	if (!running) {
	char temp[MAX_PATH];
	UpdateData(TRUE);
	m_movie="";
	UpdateData(false);
 	CFileDialog OpenMovie(true, NULL, NULL, NULL, "AVI Files (*.avi)|*.avi", NULL);
	if (OpenMovie.DoModal()==IDOK) {
		strcpy(temp, OpenMovie.GetPathName());
		m_movie = temp;
		TargetFile = OpenMovie.GetFileTitle();
		UpdateData(FALSE);
	}
	UpdateData(false);
	}
}

void COggMuxDlg::OnBOpensoundtrack1() 
{
	if (!running) {
	UpdateData(TRUE);	
 	m_soundtrack1="";
	UpdateData(false);
	FetchFile(&m_soundtrack1, "OGG Vorbis Files (*.ogg)|*.ogg|(ac3-) WAV Files(*.wav) [Quality=0 means ac3]|*.wav|MP3 Soundtrack Files (*.mp3)|*.mp3|AAC Audio Files (*.aac)|*.aac");
	OnUpdateESoundtrack1();
	}
}

void COggMuxDlg::OnBSavetargetfile() 
{
	if (!running) 
	{
		char temp[MAX_PATH];
		UpdateData(TRUE);
		m_targetfile="";
		UpdateData(false);
 		CFileDialog OpenTargetFile(false, ".ogm", TargetFile, NULL, "Ogg Media Files (*.ogm)|*.ogm", NULL);
		if (OpenTargetFile.DoModal()==IDOK) 
		{
			strcpy(temp, OpenTargetFile.GetPathName());
			m_targetfile = temp;
			UpdateData(FALSE);
		}
	}
}


void COggMuxDlg::OnBOpensubtitles1() 
{
	if (!running) {
	UpdateData(true);
	m_subtitles1="";
	UpdateData(false);
	FetchFile(&m_subtitles1, "SRT Files (*.srt)|*.srt");
	}
}

////////////////////////////////////////////////////////////
// Open and import a chapterlist (smartripper *INFO.txt)
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBImportChapters() 
{
	if (!running) {
	char buffer[256], temp[256] = "", parse;
	CString chapterlist = "";
	ifstream chapterfile;
	if (automate)
		chapterlist = chapters;
	else
		FetchFile(&chapterlist, "Chapter Info Text (*.txt)|*.txt");

	if (chapterlist != "") 
	{
		chapterfile.open(chapterlist, ios::in);
// Find format to parse
		chapterfile.getline(buffer,255);
		chapterlist=buffer;
		if (chapterlist.Left(9) == "*********")  // SmartRipper
			parse='a';
		if (chapterlist.Left(9) == "CHAPTER01")  // chapterXtractor
			parse='b';

		switch (parse)
		{
		case 'a':  // SmartRipper Format
			{
// Find the chapter names in file
				while ((chapterlist != "Chap#=Frame# Name") && (!chapterfile.eof()))
				{
					chapterfile.getline(buffer,255);
					chapterlist=buffer;
				}
// Chapternames found, now read them
				while ((chapterlist != "") && (!chapterfile.eof()))
				{
					chapterfile.getline(buffer,255);
					chapterlist=buffer;
					if (chapterlist != "")
					{
						strcpy(temp, chapterlist.Right(chapterlist.GetLength()-chapterlist.Find(" ",0)-1));
						chapterlist=temp;
						ChapterName.AddTail(chapterlist);
					}
				}
// Find the chapter times in file
				while ((chapterlist != "Chap#=Time") && (!chapterfile.eof()))
				{
					chapterfile.getline(buffer,255);
					chapterlist=buffer;
				}
// Chaptertimes found, now read them
				while ((chapterlist != "") && (!chapterfile.eof()))
				{
					chapterfile.getline(buffer,255);
					chapterlist=buffer;
					if (chapterlist != "")
					{
						strcpy(temp, chapterlist.Right(chapterlist.GetLength()-chapterlist.Find("=",0)-1));
						strcat(temp, "0");
						chapterlist=temp;
						ChapterTime.AddTail(chapterlist);
					}
				}
			}
			break;  // end of SmartRipper Parser

		case 'b':  // ChapterXtractor-Format
			{
				chapterlist=buffer;
				strcpy(temp, chapterlist.Right(chapterlist.GetLength()-chapterlist.Find("=",0)-1));
				chapterlist=temp;
				if (chapterlist.GetLength() < 9)  // old format
					strcat(temp, ".000");
				chapterlist=temp;
				ChapterTime.AddTail(chapterlist);
				chapterfile.getline(buffer,255);
				chapterlist=buffer;
				strcpy(temp, chapterlist.Right(chapterlist.GetLength()-chapterlist.Find("=",0)-1));
				chapterlist=temp;
				ChapterName.AddTail(chapterlist);
				while ((chapterlist != "") && (!chapterfile.eof()))
				{
					chapterfile.getline(buffer,255);
					chapterlist=buffer;
					if (chapterlist != "")
					{
						strcpy(temp, chapterlist.Right(chapterlist.GetLength()-chapterlist.Find("=",0)-1));
						chapterlist=temp;
						if (chapterlist.GetLength() < 9)  // old format
							strcat(temp, ".000");
						chapterlist=temp;
						ChapterTime.AddTail(chapterlist);
					}
					chapterfile.getline(buffer,255);
					chapterlist=buffer;
					if (chapterlist != "")
					{
						strcpy(temp, chapterlist.Right(chapterlist.GetLength()-chapterlist.Find("=",0)-1));
						chapterlist=temp;
						ChapterName.AddTail(chapterlist);
					}
				}
			}
			break;  // end of ChapterXtractor-Format

		default:
			{
				MessageBox("Unsupported chapterfile format!", "Chapter import error", MB_OK|MB_ICONEXCLAMATION);
			}
			break;  // end of default (unknown file format)
		} // of switch

		chapterfile.close();
		build_chapter_list();
	}
	}
}

////////////////////////////////////////////////////////////
// Make sure to remember all values
////////////////////////////////////////////////////////////
void COggMuxDlg::OnSelchangeComboLanguage1() 
{
	if (!running) {
	UpdateData(true);
	}
}

void COggMuxDlg::OnSelchangeComboLanguage2() 
{
	if (!running) {
	UpdateData(true);
	}
}

////////////////////////////////////////////////////////////
// ...add a soundtrack
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBAddsoundtrack() 
{
	if (!running) {
	char temp[MAX_PATH];
	UpdateData(true);
	SndTrk.AddTail(m_soundtrack1);
	SndTrkLng.AddTail(m_language1);
	wsprintf(temp, "%i", m_quality);
	SndTrkQual.AddTail(temp);
	wsprintf(temp, "%i", m_soundtrack_delay);
	SndTrkDelay.AddTail(temp);
	UpdateData(false);
	build_soundtrack_list();
	}
}

////////////////////////////////////////////////////////////
// ...add a subtitle
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBAddsubtitles() 
{
	if (!running) {
	char temp[MAX_PATH];
	UpdateData(true);
	Subt.AddTail(m_subtitles1);
	SubtLng.AddTail(m_language2);
	wsprintf(temp, "%i", m_subtitles_delay);
	SubtDelay.AddTail(temp);
	UpdateData(false);
	build_subtitles_list();
	}
}

////////////////////////////////////////////////////////////
// ...add a chapter
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBAddchapter() 
{
	if (!running) {
	UpdateData(true);
	ChapterName.AddTail(m_chaptername);
	ChapterTime.AddTail(m_chaptertime);
	UpdateData(false);
	build_chapter_list();
	}
}

////////////////////////////////////////////////////////////
// selection-changes in the listboxes...
////////////////////////////////////////////////////////////
void COggMuxDlg::OnSelchangeListSoundtracks() 
{
	if (!running) {
	set_soundtrack(m_soundtracklist.GetCurSel());	
	m_soundtrack_modify.EnableWindow(true);
	m_soundtrack_delete.EnableWindow(true);
	}
}

void COggMuxDlg::OnSelchangeListSubtitles() 
{
	if (!running) {
	set_subtitles(m_subtitleslist.GetCurSel());
	m_subtitles_modify.EnableWindow(true);
	m_subtitles_delete.EnableWindow(true);
	}
}

void COggMuxDlg::OnSelchangeListChapters() 
{
	if (!running) {
	set_chapter(m_chapterlist.GetCurSel());
	m_chapter_delete.EnableWindow(true);
	m_chapter_modify.EnableWindow(true);
	}
}
////////////////////////////////////////////////////////////
// Enable/disable quality-control according to filetype
////////////////////////////////////////////////////////////
void COggMuxDlg::OnUpdateESoundtrack1() 
{
	if (!running) {
	if (m_soundtrack1.Right(4)==".wav") {
		m_static_quality.EnableWindow(true);
		m_edit_quality.EnableWindow(true);
	} else {
		m_static_quality.EnableWindow(false);
		m_edit_quality.EnableWindow(false);
	}
	}
}

////////////////////////////////////////////////////////////
// Reset all values
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBReset() 
{
	if (!running) {
	m_soundtrack1 = _T("");
	m_targetfile = _T("");
	m_movie = _T("");
	m_language1 = _T("(not specified)");
	m_language2 = _T("(not specified)");
	m_subtitles1 = _T("");
	m_quality = 1;
	m_chaptername = _T("");
	m_chaptertime = _T("");
	SndTrk.RemoveAll();
	SndTrkLng.RemoveAll();
	SndTrkQual.RemoveAll();
	Subt.RemoveAll();
	SubtLng.RemoveAll();
	ChapterName.RemoveAll();
	ChapterTime.RemoveAll();
	m_soundtracklist.ResetContent();
	m_subtitleslist.ResetContent();
	m_chapterlist.ResetContent();
	m_static_quality.EnableWindow(false);
	m_edit_quality.EnableWindow(false);
	m_soundtrack_modify.EnableWindow(false);
	m_soundtrack_delete.EnableWindow(false);
	m_subtitles_modify.EnableWindow(false);
	m_subtitles_delete.EnableWindow(false);
	m_chapter_modify.EnableWindow(false);
	m_chapter_delete.EnableWindow(false);
	m_clear_chapterlist.EnableWindow(false);
	int low, high;
	m_progress.GetRange(low, high);
	m_progress.SetPos(low);
	m_static_position="";
	m_static_position2="";
	m_splitsize=0;
	UpdateData(false);
	}
}

////////////////////////////////////////////////////////////
// Set soundtrack
////////////////////////////////////////////////////////////
void COggMuxDlg::set_soundtrack(int number)
{
	if (!running) {
	m_soundtrack1 = "";
	m_language1 = "(not specified)";
	m_soundtrack_delay = 0;
	m_quality = 1;
	UpdateData(false);
	m_soundtrack1 = SndTrk.GetAt(get_soundtrack_pos(number));
	m_language1 = SndTrkLng.GetAt(get_soundlang_pos(number));
	m_quality = atoi(SndTrkQual.GetAt(get_soundqual_pos(number))); 
	m_soundtrack_delay = atoi(SndTrkDelay.GetAt(get_sounddelay_pos(number)));
	OnUpdateESoundtrack1();
	UpdateData(false);
	}
}

////////////////////////////////////////////////////////////
// Set subtitles
////////////////////////////////////////////////////////////
void COggMuxDlg::set_subtitles(int number)
{
	if (!running) {
	m_subtitles1 = "";
	m_language2 = "(not specified)";
	m_subtitles_delay = 0;
	UpdateData(false);
	m_subtitles1 = Subt.GetAt(get_subtitles_pos(number));
	m_language2 = SubtLng.GetAt(get_subtlang_pos(number));
	m_subtitles_delay = atoi(SubtDelay.GetAt(get_subtdelay_pos(number)));
	UpdateData(false);
	}
}

////////////////////////////////////////////////////////////
// Set chapter
////////////////////////////////////////////////////////////
void COggMuxDlg::set_chapter(int number)
{
	if (!running) {
	m_chaptername = "";
	m_chaptertime = "";
	UpdateData(false);
	m_chaptername = ChapterName.GetAt(get_chaptername_pos(number));
	m_chaptertime = ChapterTime.GetAt(get_chaptertime_pos(number));
	UpdateData(false);
	}
}

////////////////////////////////////////////////////////////
// Get soundtrack position
////////////////////////////////////////////////////////////
POSITION COggMuxDlg::get_soundtrack_pos(int number)
{
	POSITION currentSndTrk = SndTrk.GetHeadPosition();
	int m_Pos = 1;
	SndTrk.GetHead();
	while (m_Pos <= number) {
		if (currentSndTrk != SndTrk.GetTailPosition()) {
			SndTrk.GetNext(currentSndTrk);
		}
		m_Pos++;
	}
	return currentSndTrk;
}

POSITION COggMuxDlg::get_soundlang_pos(int number)
{
	POSITION currentSndLng = SndTrkLng.GetHeadPosition();
	int m_Pos = 1;
	SndTrkLng.GetHead();
	while (m_Pos <= number) {
		if (currentSndLng!=SndTrkLng.GetTailPosition()) {
			SndTrkLng.GetNext(currentSndLng);
		}
		m_Pos++;
	}
	return currentSndLng;
}

POSITION COggMuxDlg::get_soundqual_pos(int number)
{
	POSITION currentSndQual = SndTrkQual.GetHeadPosition();
	int m_Pos = 1;
	SndTrkQual.GetHead();
	while (m_Pos <= number) {
		if (currentSndQual!=SndTrkQual.GetTailPosition()) {
			SndTrkQual.GetNext(currentSndQual);
		}
		m_Pos++;
	}
	return currentSndQual;
}

POSITION COggMuxDlg::get_sounddelay_pos(int number)
{
	POSITION currentSndTrk = SndTrkDelay.GetHeadPosition();
	int m_Pos = 1;
	SndTrkDelay.GetHead();
	while (m_Pos <= number) {
		if (currentSndTrk!=SndTrkDelay.GetTailPosition()) {
			SndTrkDelay.GetNext(currentSndTrk);
		}
		m_Pos++;
	}
	return currentSndTrk;
}

////////////////////////////////////////////////////////////
// Get subtitles position
////////////////////////////////////////////////////////////
POSITION COggMuxDlg::get_subtitles_pos(int number)
{
	POSITION currentSubt = Subt.GetHeadPosition();
	int m_Pos = 1;
	Subt.GetHead();
	while (m_Pos <= number) {
		if (currentSubt!=Subt.GetTailPosition()) {
			Subt.GetNext(currentSubt);
		}
		m_Pos++;
	}
	return currentSubt;
}

POSITION COggMuxDlg::get_subtlang_pos(int number)
{
	POSITION currentSubt = SubtLng.GetHeadPosition();
	int m_Pos = 1;
	SubtLng.GetHead();
	while (m_Pos <= number) {
		if (currentSubt!=SubtLng.GetTailPosition()) {
			SubtLng.GetNext(currentSubt);
		}
		m_Pos++;
	}
	return currentSubt;
}

POSITION COggMuxDlg::get_subtdelay_pos(int number)
{
	POSITION currentSubt = SubtDelay.GetHeadPosition();
	int m_Pos = 1;
	SubtDelay.GetHead();
	while (m_Pos <= number) {
		if (currentSubt!=SubtDelay.GetTailPosition()) {
			SubtDelay.GetNext(currentSubt);
		}
		m_Pos++;
	}
	return currentSubt;
}

////////////////////////////////////////////////////////////
// Get chapter position
////////////////////////////////////////////////////////////
POSITION COggMuxDlg::get_chaptername_pos(int number)
{
	POSITION currentChapterName = ChapterName.GetHeadPosition();
	int m_Pos = 1;
	ChapterName.GetHead();
	while (m_Pos <= number) {
		if (currentChapterName!=ChapterName.GetTailPosition()) {
			ChapterName.GetNext(currentChapterName);
		}
		m_Pos++;
	}
	return currentChapterName;
}

POSITION COggMuxDlg::get_chaptertime_pos(int number)
{
	POSITION currentChapterTime = ChapterTime.GetHeadPosition();
	int m_Pos = 1;
	ChapterTime.GetHead();
	while (m_Pos <= number) {
		if (currentChapterTime!=ChapterTime.GetTailPosition()) {
			ChapterTime.GetNext(currentChapterTime);
		}
		m_Pos++;
	}
	return currentChapterTime;
}

////////////////////////////////////////////////////////////
// Modify soundtrack
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBSoundtrackModify() 
{
	char temp[MAX_PATH];
	UpdateData(TRUE);
	strcpy(temp, m_soundtrack1);
	SndTrk.SetAt(get_soundtrack_pos(m_soundtracklist.GetCurSel()), temp);
	strcpy(temp, m_language1);
	SndTrkLng.SetAt(get_soundlang_pos(m_soundtracklist.GetCurSel()), temp);
	wsprintf(temp, "%i", m_quality);
	SndTrkQual.SetAt(get_soundqual_pos(m_soundtracklist.GetCurSel()), temp);
	wsprintf(temp, "%i", m_soundtrack_delay);
	SndTrkDelay.SetAt(get_sounddelay_pos(m_soundtracklist.GetCurSel()), temp);
	UpdateData(false);
	build_soundtrack_list();
	m_soundtrack_modify.EnableWindow(false);
	m_soundtrack_delete.EnableWindow(false);
}

////////////////////////////////////////////////////////////
// Modify subtitles
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBSubtitleModify() 
{
	char temp[MAX_PATH];
	UpdateData(true);
	strcpy(temp, m_subtitles1);
	Subt.SetAt(get_subtitles_pos(m_subtitleslist.GetCurSel()), temp);
	strcpy(temp, m_language2);
	SubtLng.SetAt(get_subtlang_pos(m_subtitleslist.GetCurSel()), temp);
	wsprintf(temp, "%i", m_subtitles_delay);
	SubtDelay.SetAt(get_subtdelay_pos(m_soundtracklist.GetCurSel()), temp);
	UpdateData(false);
	build_subtitles_list();
	m_subtitles_modify.EnableWindow(false);
	m_subtitles_delete.EnableWindow(false);
}

////////////////////////////////////////////////////////////
// Modify chapter
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBChapterModify() 
{
	char temp[MAX_PATH];
	UpdateData(true);
	strcpy(temp, m_chaptername);
	ChapterName.SetAt(get_chaptername_pos(m_chapterlist.GetCurSel()), temp);
	strcpy(temp, m_chaptertime);
	ChapterName.SetAt(get_chaptertime_pos(m_chapterlist.GetCurSel()), temp);
	UpdateData(false);
	build_chapter_list();
	m_chapter_modify.EnableWindow(false);
	m_chapter_delete.EnableWindow(false);
}

////////////////////////////////////////////////////////////
// Delete soundtrack
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBSoundtrackDelete() 
{
	if (SndTrk.GetCount()>0) {
		SndTrk.RemoveAt(get_soundtrack_pos(m_soundtracklist.GetCurSel()));
		SndTrkLng.RemoveAt(get_soundlang_pos(m_soundtracklist.GetCurSel()));
		SndTrkQual.RemoveAt(get_soundqual_pos(m_soundtracklist.GetCurSel()));
		SndTrkDelay.RemoveAt(get_sounddelay_pos(m_soundtracklist.GetCurSel()));
		build_soundtrack_list();
		m_soundtrack_modify.EnableWindow(false);
		m_soundtrack_delete.EnableWindow(false);
	}
}

////////////////////////////////////////////////////////////
// Delete subtitles
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBSubtitleDelete() 
{
	if (Subt.GetCount()>0) {
		Subt.RemoveAt(get_subtitles_pos(m_subtitleslist.GetCurSel()));
		SubtLng.RemoveAt(get_subtlang_pos(m_subtitleslist.GetCurSel()));
		SubtDelay.RemoveAt(get_subtdelay_pos(m_subtitleslist.GetCurSel()));
		build_subtitles_list();
		m_subtitles_modify.EnableWindow(false);
		m_subtitles_delete.EnableWindow(false);
	}
}

////////////////////////////////////////////////////////////
// Delete chapter
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBChapterDelete() 
{
	if (ChapterName.GetCount()>0) {
		ChapterName.RemoveAt(get_chaptername_pos(m_chapterlist.GetCurSel()));
		ChapterTime.RemoveAt(get_chaptertime_pos(m_chapterlist.GetCurSel()));
		build_chapter_list();
		m_chapter_modify.EnableWindow(false);
		m_chapter_delete.EnableWindow(false);
	}
}

////////////////////////////////////////////////////////////
// Build soundtrack listbox
////////////////////////////////////////////////////////////
void COggMuxDlg::build_soundtrack_list()
{
	char temp[MAX_PATH];
	m_soundtracklist.ResetContent();
	for (int c=0; c<SndTrk.GetCount();c++) {
		set_soundtrack(c);
		m_edit_quality.IsWindowEnabled()?wsprintf(temp, "%i. [%s][delay: %i][qual. %d] %s", m_soundtracklist.GetCount()+1, m_language1, m_soundtrack_delay, m_quality, m_soundtrack1):wsprintf(temp, "%i. [%s][delay: %i] %s", m_soundtracklist.GetCount()+1, m_language1, m_soundtrack_delay, m_soundtrack1);
		m_soundtracklist.AddString(temp);
	}
	m_soundtrack1 = "";
	m_language1 = "(not specified)";
	m_soundtrack_delay=0;
	m_quality=1;
	OnUpdateESoundtrack1();
	UpdateData(false);
}

////////////////////////////////////////////////////////////
// Build subtitles listbox
////////////////////////////////////////////////////////////
void COggMuxDlg::build_subtitles_list()
{
	char temp[MAX_PATH];
	m_subtitleslist.ResetContent();
	for (int c=0; c<Subt.GetCount();c++) {
		set_subtitles(c);
		wsprintf(temp, "%i. [%s][delay: %i] %s", m_subtitleslist.GetCount()+1, m_language2, m_subtitles_delay, m_subtitles1);
		m_subtitleslist.AddString(temp);
	}
	m_subtitles1 = "";
	m_language2 = "(not specified)";
	m_subtitles_delay = 0;
	UpdateData(false);
}

////////////////////////////////////////////////////////////
// Build chapter listbox
////////////////////////////////////////////////////////////
void COggMuxDlg::build_chapter_list()
{
	char temp[MAX_PATH];
	m_chapterlist.ResetContent();
	for (int c=0; c<ChapterName.GetCount();c++) {
		set_chapter(c);
		wsprintf(temp, "%i. %s - %s", m_chapterlist.GetCount()+1, m_chaptername, m_chaptertime);
		m_chapterlist.AddString(temp);
	}
	m_clear_chapterlist.EnableWindow(true);
	m_chaptername = "";
	m_chaptertime = "";
	UpdateData(false);
}

////////////////////////////////////////////////////////////
// reset chapter listbox and stringlists
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBClearChapterlist() 
{
	m_chapterlist.ResetContent();
	ChapterName.RemoveAll();
	ChapterTime.RemoveAll();
	m_clear_chapterlist.EnableWindow(false);
	m_chapter_modify.EnableWindow(false);
	m_chapter_delete.EnableWindow(false);
	m_chaptername = "";
	m_chaptertime = "";
	UpdateData(false);
}

////////////////////////////////////////////////////////////
// Adding ToolTips
////////////////////////////////////////////////////////////
COggMuxDlg::PreTranslateMessage(MSG *pMsg)
{
	m_ToolTip.RelayEvent(pMsg);
	return CDialog::PreTranslateMessage(pMsg);
}

////////////////////////////////////////////////////////////
// Adding Command Line support/automisation
////////////////////////////////////////////////////////////
void COggMuxDlg::automisation(CString setupfile)
{
	char buffer[256], temp[256] = "";
	CString my_var = "";
	ifstream automisationfile;
	
	automisationfile.open(setupfile, ios::in);
	if (!automisationfile.is_open())
	{
		MessageBox("Could not open automisation file!");
		OnCancel();
	} else {
	automisationfile.getline(buffer,255);
	my_var=buffer;
	if (my_var == "<movie>")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
		m_movie = my_var;
		UpdateData(false);
		automisationfile.getline(buffer,255);
		my_var=buffer;
		if (my_var != "</movie>")
		{
			MessageBox("Error in automisation setup file, movie section corrupt!", "Automisation Error", MB_OK);
			automisationfile.close();
			OnCancel();
		}
	}

	automisationfile.getline(buffer,255);
	my_var=buffer;

	while (my_var == "")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
	}

	if (my_var == "<title>")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
		if (my_var != "</title>")
		{
			m_movieinfo.m_title = my_var;
			m_movieinfo.UpdateData(false);
			automisationfile.getline(buffer,255);
			my_var=buffer;
		}
	}

	automisationfile.getline(buffer,255);
	my_var=buffer;

	while (my_var == "")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
	}

	if (my_var == "<author>")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
		if (my_var != "</author>")
		{
			m_movieinfo.m_author = my_var;
			m_movieinfo.UpdateData(false);
			automisationfile.getline(buffer,255);
			my_var=buffer;
		}
	}

	automisationfile.getline(buffer,255);
	my_var=buffer;

	while (my_var == "")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
	}

	if (my_var == "<copyright>")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
		if (my_var != "</copyright>")
		{
			m_movieinfo.m_copyright = my_var;
			m_movieinfo.UpdateData(false);
			automisationfile.getline(buffer,255);
			my_var=buffer;
		}
	}

	automisationfile.getline(buffer,255);
	my_var=buffer;

	while (my_var == "")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
	}

	if (my_var == "<description>")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
		if (my_var != "</description>")
		{
			m_movieinfo.m_description = my_var;
			m_movieinfo.UpdateData(false);
			automisationfile.getline(buffer,255);
			my_var=buffer;
		}
	}


	automisationfile.getline(buffer,255);
	my_var=buffer;

	while (my_var == "")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
	}

	if (my_var == "<soundtracks>")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
		while (my_var != "</soundtracks>")
		{
			m_soundtrack1 = my_var;
			UpdateData(false);
			automisationfile.getline(buffer,255);
			my_var=buffer;
			m_language1 = my_var;
			UpdateData(false);
			if (m_soundtrack1.Right(3)=="wav")
			{
				automisationfile.getline(buffer,255);
				my_var=buffer;
				m_quality=atoi(my_var);
				UpdateData(false);
			}
			OnBAddsoundtrack();
			automisationfile.getline(buffer,255);
			my_var=buffer;
		}
	}

	automisationfile.getline(buffer,255);
	my_var=buffer;

	while (my_var == "")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
	}

	if (my_var == "<subtitles>")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
		while (my_var != "</subtitles>")
		{
			m_subtitles1 = my_var;
			UpdateData(false);
			automisationfile.getline(buffer,255);
			my_var=buffer;
			m_language2 = my_var;
			UpdateData(false);
			OnBAddsubtitles();
			automisationfile.getline(buffer,255);
			my_var=buffer;
		}
	}

	automisationfile.getline(buffer,255);
	my_var=buffer;

	while (my_var == "")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
	}

	if (my_var == "<chapters>")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
		if (my_var != "</chapters>")
		{
			chapters = my_var;
			UpdateData(false);
			OnBImportChapters();
			automisationfile.getline(buffer,255);
			my_var=buffer;
		}
	}

	automisationfile.getline(buffer,255);
	my_var=buffer;

	while (my_var == "")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
	}

	if (my_var == "<target>")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
		if (my_var != "</target>")
		{
			m_targetfile = my_var;
			UpdateData(false);
			automisationfile.getline(buffer,255);
			my_var=buffer;
		}
	}

	automisationfile.getline(buffer,255);
	my_var=buffer;

	while (my_var == "")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
	}

	if (my_var == "<split>")
	{
		automisationfile.getline(buffer,255);
		my_var=buffer;
		if (my_var != "</split>")
		{
			m_splitsize = atoi(my_var);
			UpdateData(false);
			automisationfile.getline(buffer,255);
			my_var=buffer;
		}
	}

	UpdateData(false);
	automisationfile.close();
	OnBMuxfile();
	} // end of if file not exists
	OnOK();
}


////////////////////////////////////////////////////////////
// Show About-Box
////////////////////////////////////////////////////////////
void COggMuxDlg::OnBAbout() 
{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
}

////////////////////////////////////////////////////////////
// AboutBox handling
////////////////////////////////////////////////////////////
void CAboutDlg::OnOK() 
{
	CDialog::OnOK();
}

////////////////////////////////////////////////////////////
// MovieInfo calling
////////////////////////////////////////////////////////////
void COggMuxDlg::OnMovieinfo() 
{
	m_movieinfo.DoModal();
}
